package io.gitlab.jfronny.scripts

import org.gradle.api.Action
import org.gradle.api.Project
import org.gradle.api.provider.ListProperty
import org.gradle.api.provider.Property
import org.gradle.api.tasks.Nested

interface JfModExtension: LomExtension {
    val libJfVersion: Property<String>
    @get:Nested val curseforge: CurseForge
    @get:Nested val modrinth: Modrinth

    override fun check(proj: Project) {
        super.check(proj)
        libJfVersion.finalizeValue()
        val p = proj.allprojects
        modrinth.check(proj)
        curseforge.check(proj)
    }

    fun isMod(proj: Project) = proj.extensions.findByName("jfModule") != null
    fun curseforge(closure: Action<CurseForge>) = closure.execute(curseforge)
    fun modrinth(closure: Action<Modrinth>) = closure.execute(modrinth)

    interface CurseForge {
        val projectId: Property<String>
        val requiredDependencies: ListProperty<String>
        val optionalDependencies: ListProperty<String>

        fun check(proj: Project) {
            projectId.finalizeValue()
            requiredDependencies.finalizeValue()
            optionalDependencies.finalizeValue()
            if (!projectId.isPresent) {
                require(requiredDependencies.map { it.isEmpty() }.getOrElse(true)) { "curseforge is not configured for a project id but has dependencies" }
                require(optionalDependencies.map { it.isEmpty() }.getOrElse(true)) { "curseforge is not configured for a project id but has dependencies" }
            }
        }
    }

    interface Modrinth {
        val projectId: Property<String>
        val requiredDependencies: ListProperty<String>
        val optionalDependencies: ListProperty<String>

        fun check(proj: Project) {
            projectId.finalizeValue()
            requiredDependencies.finalizeValue()
            optionalDependencies.finalizeValue()
            if (!projectId.isPresent) {
                require(requiredDependencies.map { it.isEmpty() }.getOrElse(true)) { "modrinth is not configured for a project id but has dependencies" }
                require(optionalDependencies.map { it.isEmpty() }.getOrElse(true)) { "modrinth is not configured for a project id but has dependencies" }
            }
        }
    }
}